MODULE Blinker;
(* =========================================================================  
   Example Oberon ARM Cortex-M7 Program  
 
   Description:
     Led connected to Port A5 blinking approx. once per second

   Target:
     STM-32F334
     
   Tested on: 
     STM32F334 Nucleo-64 Board
   
   References: 
     The Programming Language Oberon 
     Oberon for Cortex-M4 Microcontrollers
     STMicroelectronics STM32 Nucleo-64 board User manual UM1724
     STMicroelectronics STM32F334xx Reference Manual RM0364
   
   (c) 2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT GPIO, Main, MCU, SYSTEM, Timers;

PROCEDURE ConfigureOutput(base, no: INTEGER);
VAR
  config: GPIO.PortConfiguration;
BEGIN
  config.mode := GPIO.Mode_Out;
  config.speed := GPIO.Speed_100MHz;
  config.outputType := GPIO.Push_Pull;
  config.resistors := GPIO.Floating;
  config.alternateFunction := GPIO.AF0; 
  GPIO.Configure(base, no, config);
END ConfigureOutput;
  
PROCEDURE Run();
VAR
  ahbenr: SET;
  msecTimer: Timers.Timer;
BEGIN
  (* enable clock for GPIO-A *)
  SYSTEM.GET(MCU.RCC_AHBENR, ahbenr);
  SYSTEM.PUT(MCU.RCC_AHBENR, ahbenr + {MCU.RCC_AHBENR_GPIOA});
  (* Green LED is connected to GPIO port A5 *)
  ConfigureOutput(MCU.GPIOABase, 5);
  (* Millisecond timer *)
  Timers.Init(msecTimer, Timers.TIM3, Timers.MSecs);
  
  WHILE TRUE DO
    GPIO.Set(MCU.GPIOABase, 5, TRUE);
    Timers.Delay(msecTimer, 500);
    GPIO.Set(MCU.GPIOABase, 5, FALSE);
    Timers.Delay(msecTimer, 500)
  END
END Run;

BEGIN
  Run()
END Blinker.
