(* =========================================================================  
   Astrobe Library Functions for STM32 Caches 
   Target: Cortex-M7 (STM32F7)
   
   (c) 2016 weControl   
   http://www.wecontrol.ch  
   
   ========================================================================= *)

MODULE GPIO;

IMPORT SYSTEM, MCU;

CONST
  Mode_In* = 0;
  Mode_Out* = 1;
  Mode_AF* = 2;
  Mode_Analog* = 3;
  
  Push_Pull* = 0;
  Open_Drain* = 1;
  
  Speed_2MHz* = 0;
  Speed_25MHz* = 1;
  Speed_50MHz* = 2;
  Speed_100MHz* = 3;
  
  Floating* = 0;
  Pull_Up* = 1;
  Pull_Down* = 2;
  
  AF0* = 0;
  AF1* = 1;
  AF2* = 2;
  AF3* = 3;
  AF4* = 4;
  AF5* = 5;
  AF6* = 6;
  AF7* = 7;
  AF8* = 8;
  AF9* = 9;
  AF10* = 10;
  AF11* = 11;
  AF12* = 12;
  AF13* = 13;
  AF14* = 14;
  AF15* = 15;
  
  GPIO_MODER = 000H;
  GPIO_OTYPER = 004H;
  GPIO_OSPEEDR = 008H;
  GPIO_PUPDR = 00CH;
  GPIO_IDR = 010H;
  GPIO_ODR = 014H;
  GPIO_BSRR = 018H;
  GPIO_LCKR = 01CH;
  GPIO_AFRL = 020H;
  GPIO_AFRH = 024H;

TYPE
  PortConfiguration* = RECORD
    mode*: INTEGER;
    outputType*: INTEGER;
    speed*: INTEGER;
    resistors*: INTEGER;
    alternateFunction*: INTEGER
  END;

PROCEDURE Configure*(base, no: INTEGER; config: PortConfiguration);
VAR
  p1, p2, p4: INTEGER;
  m1, m2, m4, x: SET;
BEGIN
  p1 := no;
  p2 := no * 2;
  p4 := (no * 4) MOD 32;
  m1 := BITS(p1);
  m2 := BITS(LSL(03H, p2));
  m4 := BITS(LSL(0FH, p4));
  SYSTEM.GET(base + GPIO_MODER, x);
  SYSTEM.PUT(base + GPIO_MODER, x - m2 + BITS(LSL(config.mode, p2)));
  SYSTEM.GET(base + GPIO_OTYPER, x);
  SYSTEM.PUT(base + GPIO_OTYPER, x - m1 + BITS(LSL(config.outputType, p1)));
  SYSTEM.GET(base + GPIO_OSPEEDR, x);
  SYSTEM.PUT(base + GPIO_OSPEEDR, x - m2 + BITS(LSL(config.speed, p2)));
  SYSTEM.GET(base + GPIO_PUPDR, x);
  SYSTEM.PUT(base + GPIO_PUPDR, x - m2 + BITS(LSL(config.resistors, p2)));
  IF no < 8 THEN
    SYSTEM.GET(base + GPIO_AFRL, x);
    SYSTEM.PUT(base + GPIO_AFRL, x - m4 + BITS(LSL(config.alternateFunction, p4)))
  ELSE
    SYSTEM.GET(base + GPIO_AFRH, x);
    SYSTEM.PUT(base + GPIO_AFRH, x - m4 + BITS(LSL(config.alternateFunction, p4)))
  END
END Configure;

PROCEDURE* Set*(base, no: INTEGER; state: BOOLEAN);
BEGIN
  IF state THEN
    SYSTEM.PUT(base + GPIO_BSRR, {no})
  ELSE
    SYSTEM.PUT(base + GPIO_BSRR, {no + 16})
  END
END Set;

END GPIO.


