MODULE Info;
(* =========================================================================  
   Example ARM Cortex-M Oberon Program  
 
   Description:
     Displays the linker and startup parameters used for the current 
     application.
  
   Target:
     All supported Cortex-M systems
     
   References: 
     The Programming Language Oberon
     Oberon for Cortex-M Microcontrollers
   
   (c) 2012-2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)


IMPORT LinkOptions, Main, MCU, Out, Storage;

PROCEDURE OutTitle(title: ARRAY OF CHAR);
CONST
  width = 20;
VAR
  i: INTEGER;
BEGIN
  Out.String(title);
  FOR i := LEN(title) + 1 TO width DO Out.Char(" ") END 
END OutTitle;

PROCEDURE OutFreq(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Int(item, 9);
  Out.String(" Hz");
  Out.Ln
END OutFreq;

PROCEDURE OutAddr(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Hex(item, 0);
  Out.Ln
END OutAddr;

PROCEDURE OutSize(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Int(item, 0);
  Out.String(" Bytes");
  Out.Ln
END OutSize;

PROCEDURE Run();
BEGIN
  OutFreq("Crystal Frequency:", LinkOptions.Fosc);
  OutFreq("CPU Clock:", MCU.CCLK);
  OutFreq("Peripheral Clock:", MCU.PCLK);
  OutAddr("Data Start:", LinkOptions.DataStart);
  OutAddr("Data End:", LinkOptions.DataEnd);
  OutAddr("Code Start:", LinkOptions.CodeStart);
  OutAddr("Code End:", LinkOptions.CodeEnd);
  OutAddr("Heap Start:", Storage.HeapStart());
  OutAddr("Heap Pointer:", Storage.HeapPtr());
  OutSize("Heap Available:", Storage.HeapAvailable());
  OutSize("Heap Used:", Storage.HeapUsed());
  OutAddr("Stack Start:", Storage.StackStart());
  OutAddr("Stack Pointer:", Storage.StackPtr());
  OutSize("Stack Available:", Storage.StackAvailable());
  OutSize("Stack Used:", Storage.StackUsed())
END Run;

BEGIN
  Run()
END Info.

