MODULE MCU;
(* -------------------------------------------------------------------------  
   STM32F334-specific constant definitions 
 
   Target: STM32F334 family microcontrollers
   
   (c) 2017 CFB Software  
   http://www.astrobe.com  
   
   ------------------------------------------------------------------------- *)

IMPORT SYSTEM;
  
CONST
  STM32F334* = TRUE;

  (* Corresponds to the Astrobe version e.g. 630 = v6.3.0 *)
  Version* = 630;

  (* ASM Instructions *)
  DSB* = 0BFF34F8FH;
  ISB* = 0BFF3368FH;
  
  SYSCLK* = 8000000; 
  PCLK1* = SYSCLK * 4;
  PCLK2* = SYSCLK;  
  CCLK* = SYSCLK;
  PCLK* = PCLK1;
    
  RCCBase* = 040021000H;
  RCC_CR* = RCCBase + 000H;
  RCC_CFGR* = RCCBase + 004H;
  RCC_CIR* = RCCBase + 008H;
  RCC_APB2RSTR* = RCCBase + 00CH;
  RCC_APB1RSTR* = RCCBase + 010H;
  RCC_AHBENR* = RCCBase + 014H;
  RCC_APB2ENR* = RCCBase + 018H;
  RCC_APB1ENR* = RCCBase + 01CH;
  RCC_BDCR* = RCCBase + 020H;
  RCC_CSR* = RCCBase + 024H;
  RCC_AHBRSTR* = RCCBase + 028H;
  RCC_CFGR2* = RCCBase + 02CH;
  RCC_CFGR3* = RCCBase + 030H;

  GPIOABase* = 048000000H;
  GPIOBBase* = 048000400H;
  GPIOCBase* = 048000800H;
  GPIODBase* = 048000C00H;
  GPIOFBase* = 048001400H;
  
  USART2Base* = 040004400H;
  USART3Base* = 040004800H;
  
  TIM2Base* = 040000000H;
  TIM3Base* = 040000400H;
  TIM6Base* = 040001000H;
  TIM7Base* = 040001400H;
  
  FLASHBase* = 040022000H;
  FLASH_ACR* = FLASHBase + 000H;
  FLASH_KEYR* = FLASHBase + 004H;
  FLASH_OPTKEYR* = FLASHBase + 008H;
  FLASH_SR* = FLASHBase + 00CH;
  FLASH_CR* = FLASHBase + 010H;
  FLASH_AR* = FLASHBase + 014H;
  FLASH_OBR* = FLASHBase + 01CH;
  FLASH_WRPR* = FLASHBase + 020H;
  
  PWRBase* = 040007000H;
  PWR_CR* = PWRBase + 000H;
  PWR_CSR* = PWRBase + 004H;
  
  RTCBase*  = 040002800H;
  
  SPI1Base* = 040013000H;
  I2C1Base* = 040005400H;

  NVICBase* = 0E000E000H;
  NVIC_ISER* = NVICBase + 0100H;
  NVIC_ICER* = NVICBase + 0180H;
  NVIC_ISPR* = NVICBase + 0200H;
  NVIC_ICPR* = NVICBase + 0280H;
  NVIC_IABR* = NVICBase + 0300H;
  NVIC_IPR* = NVICBase + 0400H;
  STIR* = NVICBase + 0F00H;
  VTOR* = NVICBase + 0D08H;
  IRQ_USART3* = 39;
  
  RCC_APB1ENR_TIM2* = 0;
  RCC_APB1ENR_TIM3* = 1;
  RCC_APB1ENR_TIM6* = 4;
  RCC_APB1ENR_TIM7* = 5;
  RCC_APB1ENR_USART2* = 17;
  RCC_APB1ENR_USART3* = 18;
  RCC_APB1ENR_I2C1* = 21;
  RCC_APB1ENR_PWR* = 28;

  RCC_APB2ENR_SPI1* = 12;

  RCC_AHBENR_GPIOA* = 17;  
  RCC_AHBENR_GPIOB* = 18;
  RCC_AHBENR_GPIOC* = 19;
  RCC_AHBENR_GPIOD* = 20;
  RCC_AHBENR_GPIOF* = 22;
 
 
VAR
  HSECLK*: INTEGER;

PROCEDURE* Configure*();
BEGIN
  HSECLK := 8000000; (* LinkOptions.Fosc; *)
END Configure;

PROCEDURE* NVIC_EnableIRQ*(irqNo: LONGINT);
BEGIN
  SYSTEM.PUT(NVIC_ISER + (irqNo DIV 32) * 4, {irqNo MOD 32})
END NVIC_EnableIRQ;

END MCU.
