MODULE Serial;
(* =========================================================================  
   Astrobe Library Functions for Serial IO via USART2/3 
   Target: Cortex-M4 (STM32F334)
   
   (c) 2016 weControl   
   http://www.wecontrol.ch  
   (c) 2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT SYSTEM, MCU, GPIO; 

CONST
  USART2* = 2;
  USART3* = 3;
  ISR_RXNE = 5;
  ISR_TXE  = 7;


VAR
  USARTBase: INTEGER;
  CR1: INTEGER;
  CR2: INTEGER;
  CR3: INTEGER;
  BRR: INTEGER;
  ISR: INTEGER;
  RDR: INTEGER;
  TDR: INTEGER;

PROCEDURE* SetUsartNo(usartNo: INTEGER);
BEGIN
  CASE usartNo OF
  | USART2: USARTBase := MCU.USART2Base
  | USART3: USARTBase := MCU.USART3Base
  END;
  CR1 := USARTBase + 000H;
  CR2 := USARTBase + 004H;
  CR3 := USARTBase + 008H;
  BRR := USARTBase + 00CH;
  ISR := USARTBase + 01CH;
  RDR := USARTBase + 024H;
  TDR := USARTBase + 028H;
END SetUsartNo; 

    
PROCEDURE* PutCh*(ch: CHAR);
BEGIN
  REPEAT UNTIL SYSTEM.BIT(ISR, ISR_TXE);
  SYSTEM.PUT(TDR, ch)
END PutCh;


PROCEDURE* TxReady*(): BOOLEAN;
  RETURN SYSTEM.BIT(ISR, ISR_TXE)
END TxReady;


PROCEDURE* GetCh*(VAR ch: CHAR);
BEGIN
  REPEAT UNTIL SYSTEM.BIT(ISR, ISR_RXNE);
  SYSTEM.GET(RDR, ch)
END GetCh;


PROCEDURE* RxReady*(): BOOLEAN;
  RETURN SYSTEM.BIT(ISR, ISR_RXNE)
END RxReady;


PROCEDURE ConfigurePins(usartNo: INTEGER);
VAR
  apbenr, ahbenr: SET;
  config: GPIO.PortConfiguration;
BEGIN
  SYSTEM.GET(MCU.RCC_APB1ENR, apbenr);
  CASE usartNo OF
    USART2:
      INCL(apbenr, MCU.RCC_APB1ENR_USART2) |
    USART3:
      INCL(apbenr, MCU.RCC_APB1ENR_USART3) 
  END;
  SYSTEM.PUT(MCU.RCC_APB1ENR, apbenr);
  SYSTEM.GET(MCU.RCC_AHBENR, ahbenr);
  SYSTEM.PUT(MCU.RCC_AHBENR, ahbenr + {MCU.RCC_AHBENR_GPIOA, MCU.RCC_AHBENR_GPIOC});
  (* configure GPIOs *)
  config.mode := GPIO.Mode_AF;
  config.outputType := GPIO.Push_Pull;
  config.speed := GPIO.Speed_100MHz;
  config.resistors := GPIO.Pull_Up;
  config.alternateFunction := GPIO.AF7;
  CASE usartNo OF
    USART2:
      GPIO.Configure(MCU.GPIOABase, 2, config);
      GPIO.Configure(MCU.GPIOABase, 3, config) |
    USART3:
      GPIO.Configure(MCU.GPIOCBase, 10, config);
      GPIO.Configure(MCU.GPIOCBase, 11, config)
  END 
END ConfigurePins;


PROCEDURE Init*(usartNo, baudRate: INTEGER);
VAR
  intDivider, fracDivider, brr, apbClock: INTEGER;
BEGIN
  SetUsartNo(usartNo);
  ConfigurePins(usartNo);
  (* configure Usart *)
  apbClock := MCU.PCLK1;
  intDivider := (25 * apbClock) DIV (4 * baudRate);
  fracDivider := intDivider MOD 100;
  brr := ((fracDivider * 16) + 50) DIV 100 MOD 16 + (intDivider DIV 100) * 16;
  SYSTEM.PUT(BRR, brr); 
  SYSTEM.PUT(CR1, {0, 2, 3});
  SYSTEM.PUT(CR2, { });
  SYSTEM.PUT(CR3, { }); 
END Init;

END Serial.
         
